/**
 * XmlHelferlein.java
 * eu.gronos.kostenrechner.util (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Allgemein;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Dateikopf;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Streitgenossen;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Teilklageruecknahme;
import eu.gronos.kostenrechner.view.baumbach.BaumbachAssistentDialog;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Hält Methoden und Konstanten zum Lesen, Schreiben und Verwalten von
 * VerfahrensDatenContainern bereit.
 *
 * @author Peter Schuster (setrok)
 * @date 01.07.2018
 *
 */
public class VerfahrensDatenContainerUtil {

	/**
	 * Konstruktor: Standardkonstruktor ohne Parameter und ohne Bedeutung, da alle
	 * Methoden statisch sind.
	 * 
	 */
	public VerfahrensDatenContainerUtil() {
		super();
	}

	/**
	 * Die Methode dient dazu, alle Streitwertangaben auf den im
	 * {@link VerfahrensDatenContainer} angegebenen höchsten Streitwert der
	 * Streitgenossen zu setzen.
	 * 
	 * @param container ein {@link VerfahrensDatenContainer}
	 * @return den "aufgefüllten" {@link VerfahrensDatenContainer}
	 */
	private static VerfahrensDatenContainer setzeAlleStreitwerte(VerfahrensDatenContainer container) {
		final double streitwert = container.allgemein.streitwert;
		if (streitwert <= 0)
			return container;
		/* Zuerst auf Registerkarte 1 verteilen */
		Teilklageruecknahme t = container.teilklageruecknahme;
		t.streitwerteUndObsiegen = new ArrayList<Long>();
		t.streitwerteUndObsiegen.add((long) streitwert);
		t.streitwerteUndObsiegen.add((long) streitwert);
		t.streitwerteUndObsiegen.add((long) streitwert);
		/* Dann auf Registerkarte 4 */
		container.gebuehrenBerechnung.streitwert = (long) streitwert;
		/* und Rückgabe */
		return container;
	}

	/**
	 * Die Methode sucht den höchsten Streitwert aus den UnterContainern heraus und
	 * schreibt ihn in {@link Allgemein#streitwert}
	 *
	 */
	public static void setzeAllgemeinStreitwert(VerfahrensDatenContainer container) {
		container.allgemein.streitwert = Math.max(
				container.streitgenossen.klaeger.getHoechstenStreitwert(Beteiligter.KLAEGER),
				Math.max(
						container.streitgenossen.drittwiderbeklagte
								.getHoechstenStreitwert(Beteiligter.DRITTWIDERBEKLAGTE),
						Math.max(container.streitgenossen.beklagte.getHoechstenStreitwert(Beteiligter.BEKLAGTE),
								Math.max((double) container.gebuehrenBerechnung.streitwert,
										container.teilklageruecknahme.streitwerteUndObsiegen.get(0).doubleValue()))));
	}

	/**
	 * Die Methode dient dazu, die Beteiligten hinsichtlich genusNumerus anhand des
	 * Inhalts der {@link BaumbachBeteiligtenListe BaumbachBeteiligtenListen} der
	 * Streitgenossen zu setzen.
	 * 
	 * @param container ein {@link VerfahrensDatenContainer}
	 * @return den "aufgefüllten" {@link VerfahrensDatenContainer}
	 */
	private static VerfahrensDatenContainer setzeAlleBeteiligte(VerfahrensDatenContainer container) {
		/* Erst den Genus/Numerus zu ermitteln */
		BaumbachBeteiligtenListe klaegerseite = new BaumbachBeteiligtenListe();
		klaegerseite.addAll(container.streitgenossen.klaeger);
		if (container.streitgenossen.drittwiderbeklagte != null)
			klaegerseite.addAll(container.streitgenossen.drittwiderbeklagte);
		Beteiligter kl = ermittleGenusNumerus(klaegerseite, Beteiligter.KLAEGER);
		BaumbachBeteiligtenListe beklagtenseite = new BaumbachBeteiligtenListe();
		beklagtenseite.addAll(container.streitgenossen.beklagte);
		Beteiligter bk = ermittleGenusNumerus(beklagtenseite, Beteiligter.BEKLAGTE);
		ArrayList<Beteiligter> parteien = new ArrayList<Beteiligter>();
		/* Dann das Ergebnis einsetzen */
		parteien.add(kl);
		parteien.add(bk);
		container.teilklageruecknahme.parteien = parteien;
		container.staffelung.parteien = parteien;
		/* Und zurückgeben */
		return container;
	}

	/**
	 * Statische Convenience-Methode, die für Programmteile ein neues
	 * Parameterobjekt für die Verfahrensdaten erzeugt, die früher
	 * VerfahrensDatenContainer genutzt haben, vor allem für den
	 * {@link XjustizContentHandler} und den {@link BaumbachAssistentDialog}.
	 * 
	 * @param klaeger             die BaumbachBeteiligtenListe mit den Klägern
	 * @param drittwiderbeklagten die BaumbachBeteiligtenListe mit den
	 *                            Drittwiderbeklagten
	 * @param beklagten           die BaumbachBeteiligtenListe mit den Beklagten
	 * @param streitwert          den Streitwert als double
	 * @param aktenzeichen        das Aktenzeichen als String
	 */
	public static VerfahrensDatenContainer erstelleContainer(BaumbachBeteiligtenListe klaeger,
			BaumbachBeteiligtenListe drittwiderbeklagte, BaumbachBeteiligtenListe beklagte, double streitwert,
			String aktenzeichen) {
		VerfahrensDatenContainer c = new VerfahrensDatenContainer();
		/* Erst die BaumbachBeteiligtenListen einpacken */
		Streitgenossen s = c.streitgenossen;
		s.klaeger = klaeger;
		s.drittwiderbeklagte = drittwiderbeklagte;
		s.beklagte = beklagte;
		/*
		 * Vorher gab's kein Feld fürs Vorhandensein einer Widerklage, jetzt gibt's
		 * eins, dann muss es auch richtig gefüllt werden. Die Methode bleibt dieselbe.
		 */
		s.widerklage = gibtsDrittwiderbeklagte(s) || (gibtsKlaeger(c) && s.klaeger.enthaeltAnWiderklageBeteiligte())
				|| (gibtsBeklagte(c) && s.beklagte.enthaeltAnWiderklageBeteiligte());
		/* Dann die allgemeinen Teile einpacken */
		Allgemein a = c.allgemein;
		a.speicherdatum = speicherDatum();
		a.aktenzeichen = aktenzeichen;
		a.streitwert = streitwert;
		a.selectedPanel = StreitgenossenPanel.TAB_STREITGENOSSEN;
		/* Damit der Rest nicht leer ist */
		c = setzeAlleStreitwerte(setzeAlleBeteiligte(c));
		return c;
	}

	/**
	 * Die Methode dient dazu, den Genus/Numerus zu ermitteln
	 * 
	 * @param liste eine {@link BaumbachBeteiligtenListe}
	 * @param typ   {@link Beteiligter#KLAEGER} oder {@link Beteiligter#BEKLAGTE}
	 * @return einen {@link Beteiligter Beteiligten} mit dem richtigen genusNumerus:
	 *         Wenn die jeweilige Liste mehrere Beteiligte hat, ist Plural angesagt.
	 *         Wenn männliche Beteiligte in der Liste sind, dann ist die Liste
	 *         grammatikalisch männlich; wenn's nur weibliche sind, dann weiblich
	 */
	private static Beteiligter ermittleGenusNumerus(BaumbachBeteiligtenListe liste, int typ) {
		Beteiligter bt = new Beteiligter(typ, Beteiligter.MAENNLICH);
		if (liste.size() > 0 && liste.size() < 2)
			bt.setGenusNumerus(liste.getGenusNumerusFor(0));
		else {
			int genus = Beteiligter.PLURAL;
			boolean maenner = false;
			for (BaumbachBeteiligter bb : liste)
				if (bb.getGenusNumerus() == Beteiligter.MAENNLICH)
					maenner = true;
			if (!maenner)
				genus += Beteiligter.WEIBLICH;
			bt.setGenusNumerus(genus);
			/*
			 * die laufende Nummer entspricht bei den Auswahllisten an der Oberfläche dem
			 * genusNumerus
			 */
			bt.setLfdNr(genus);
		}
		return bt;
	}

	/**
	 * @param container der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn der {@link VerfahrensDatenContainer} eine
	 *         nicht-leere Klägerliste enthält.
	 */
	public static boolean gibtsKlaeger(VerfahrensDatenContainer container) {
		return container.streitgenossen.klaeger != null && !container.streitgenossen.klaeger.isEmpty();
	}

	/**
	 * @param container der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn es darin eine nicht-leere
	 *         Drittwiderbeklagten-Liste gibt
	 */
	public static boolean gibtsDrittwiderbeklagte(Streitgenossen streitgenossen) {
		return streitgenossen.drittwiderbeklagte != null && !streitgenossen.drittwiderbeklagte.isEmpty();
	}
	// container. container. VerfahrensDatenContainer container

	/**
	 * @param container der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn es darin eine nicht-leere Beklagten-Liste
	 *         gibt
	 */
	public static boolean gibtsBeklagte(VerfahrensDatenContainer container) {
		return container.streitgenossen.beklagte != null && !container.streitgenossen.beklagte.isEmpty();
	}

	/**
	 * Die Methode dient dazu, das aktuelle Systemdatum zum Speichern zu ermitteln
	 * 
	 * @return einen {@link XMLGregorianCalendar} mit dem aktuellen Systemdatum
	 * @throws DatatypeConfigurationException
	 */
	public static XMLGregorianCalendar speicherDatum() {
		try {
			DatatypeFactory factory = DatatypeFactory.newInstance();
			return factory.newXMLGregorianCalendar(new GregorianCalendar());
		} catch (DatatypeConfigurationException dce) {
			FehlerDialog.zeigeFehler("Schwerer Fehler beim Konfigurieren der Zeit!", dce);
			return null;
		}
	}

	/**
	 * Die Methode dient dazu, die Daten aus den Untercontainern {@link Dateikopf}
	 * und {@link Allgemein} zu einem String zusammenzufassen
	 * 
	 * @param container ein {@link VerfahrensDatenContainer}
	 * @return ein String
	 */
	public static String allgemeinContainerToString(VerfahrensDatenContainer container) {
		GregorianCalendar gregorianCalendar = new GregorianCalendar();
		if (container.allgemein.speicherdatum != null) {
			gregorianCalendar = container.allgemein.speicherdatum.toGregorianCalendar();
		}
		// + "\nBerechnung vom "+ ", zu Az. " +final String string = string;
		return String.format(
				"Datei-/Systeminfo [speicherdatum=%s, aktenzeichen=%s, version=%s, osName=%s, osArch=%s, osVersion=%s, javaVendor=%s, javaName=%s, javaVersion=%s]",
				DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.GERMAN).format(gregorianCalendar.getTime()),
				container.allgemein.aktenzeichen, container.dateikopf.version, container.dateikopf.osName,
				container.dateikopf.osArch, container.dateikopf.osVersion, container.dateikopf.javaVendor,
				container.dateikopf.javaName, container.dateikopf.javaVersion);
	}

}
