/*
 * FehlerHelper.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.io.IOException;
import java.io.StringWriter;

import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;

/**
 * Klasse dient dazu die Message, LocalizedMessage und den StackTrace eines
 * Fehlers in einen {@link StringBuffer} oder ein {@link HTMLDocument} zu
 * schreiben.
 *
 * @author Peter Schuster (setrok)
 * @date 24 Apr 2019
 *
 */
public class FehlerHelper {

	private static final String LIST_ITEM = "<li>%s</li>";
	private static final String UL_START = "<ul class=\"fehler\">";
	private static final String P_ELEMENT = "<p class=\"text\">%s</p>";
	private static final String H3_HEAD = "<h3 class=\"fehler\">%s</h3>";
	private Throwable throwable;

	/**
	 * @param throwable ein {@link Throwable}, um die {@link StackTraceElement}s
	 *                  auszulesen
	 */
	public FehlerHelper(Throwable throwable) {
		this.throwable = throwable;
	}

	/**
	 * Die Methode dient dazu, die Message, {@link Throwable#getLocalizedMessage()}
	 * und den {@link Throwable#getStackTrace() eines Fehlers in einen
	 * {@link StringBuffer} zu schreiben.
	 * 
	 * @return {@link Throwable#getLocalizedMessage()} und
	 *         {@link StackTraceElement}s als {@link StringBuffer}
	 */
	public StringBuffer getFehlerText() {
		StringBuffer sb = new StringBuffer();
		if (throwable != null) {
			sb.append(throwable.getMessage() + "\n");
			sb.append(throwable.getLocalizedMessage() + "\n");
			for (StackTraceElement ste : throwable.getStackTrace()) {
				sb.append("\t" + ste.toString() + "\n");
			}
		}
		return sb;
	}

	/**
	 * Die Methode schreibt die Message, {@link Throwable#getLocalizedMessage()} und
	 * den {@link Throwable#getStackTrace() eines Fehlers in das übergebene
	 * {@link HTMLDocument}.
	 * 
	 * @param doc       das {@link HTMLDocument}, in das der Text geschrieben werden
	 *                  soll.
	 * @param throwable ein {@link Throwable}, um die {@link StackTraceElement}s
	 *                  auszulesen
	 * @return das übergebene {@link HTMLDocument} mit dem eingesetzten Text aus
	 *         {@link Throwable#getLocalizedMessage()} und {@link StackTraceElement}
	 * @throws {@link BadLocationException} if the given position does not represent
	 *                a valid location in the associated document.
	 * @throws {@link IOException} TODO (kommentieren)
	 * @throws {@link IllegalArgumentException} if <code>elem</code> is a leaf
	 * @throws {@link IllegalStateException} if an HTMLEditorKit.Parser has not been
	 *                set on the document
	 */
	public HTMLDocument baueFehlerText(HTMLDocument doc, Throwable throwable)
			throws BadLocationException, IOException, IllegalArgumentException, IllegalStateException {
		if (throwable != null) {
			Element elem = baueFehlerUeberschrift(doc, throwable.getMessage(), throwable.getLocalizedMessage());
			baueStackTraceUL(doc, elem, throwable.getStackTrace());
		}
		return doc;
	}

	/**
	 * Die Methode schreibt den übergebenen {@link Throwable#getStackTrace()} in
	 * eine &lt;ul&gt; des {@link HTMLDocument}s.
	 * 
	 * @param doc        das {@link HTMLDocument}, in das der Text geschrieben
	 *                   werden soll.
	 * @param elem       das {@link Element} aus dem {@link HTMLDocument}, an dessen
	 *                   Ende die Liste eingefügt werden soll.
	 * @param stackTrace ein Array von {@link StackTraceElement}s
	 * @return das Kind-{@link Element} mit der &lt;ul&gt;
	 * @throws {@link BadLocationException} if the given position does not represent
	 *                a valid location in the associated document.
	 * @throws {@link IOException} TODO (kommentieren)
	 * @throws {@link IllegalArgumentException} if <code>elem</code> is a leaf
	 * @throws {@link IllegalStateException} if an HTMLEditorKit.Parser has not been
	 *                set on the document
	 */
	protected Element baueStackTraceUL(HTMLDocument doc, Element elem, StackTraceElement[] stackTrace)
			throws BadLocationException, IOException, IllegalArgumentException, IllegalStateException {
		doc.insertBeforeEnd(elem, UL_START);
		/* Um ins <ul> zu schreiben. */
		Element ul = elem.getElement(elem.getElementCount() - 1);
		for (StackTraceElement ste : stackTrace) {
			doc.insertBeforeEnd(ul, String.format(LIST_ITEM, ste.toString()));
		}
		return ul;
	}

	/**
	 * Die Methode schreibt die übergebenen {@link Throwable#getMessage()} und
	 * {@link Throwable#getLocalizedMessage()} in eine Überschrift des &lt;ul&gt;
	 * des {@link HTMLDocument}s.
	 * 
	 * @param doc              das {@link HTMLDocument}, in das der Text geschrieben
	 *                         werden soll.
	 * @param message          die {@link Throwable#getMessage()}, wird zu einem
	 *                         &lt;h3&gt;-Element
	 * @param localizedMessage die {@link Throwable#getLocalizedMessage()}, wird zu
	 *                         einer Unterüberschrift.
	 * @return das {@link Element}, in das die Überschrift geschrieben wurde
	 * @throws {@link BadLocationException} if the given position does not represent
	 *                a valid location in the associated document.
	 * @throws {@link IOException} TODO (kommentieren)
	 * @throws {@link IllegalArgumentException} if <code>elem</code> is a leaf
	 * @throws {@link IllegalStateException} if an HTMLEditorKit.Parser has not been
	 *                set on the document
	 */
	protected Element baueFehlerUeberschrift(HTMLDocument doc, String message, String localizedMessage)
			throws BadLocationException, IOException {
		Element elem = doc.getRootElements()[0].getElement(0);
		doc.setInnerHTML(elem, String.format(H3_HEAD, message));
		if (!message.equals(localizedMessage) && localizedMessage != null) {
			doc.insertBeforeEnd(elem, String.format(P_ELEMENT, localizedMessage));
		}
		return elem;
	}

	/**
	 * Die Methode gibt das erstellte {@link HTMLDocument} nach {@link System#err}
	 * aus.
	 * 
	 * @param doc das {@link HTMLDocument}, in das der Text geschrieben werden soll.
	 * @param kit das {@link HTMLEditorKit}, das dabei hilft.
	 * @return einen {@link String} mit dem HTML-Code
	 * @throws {@link BadLocationException} if the given position does not represent
	 *                a valid location in the associated document.
	 * @throws {@link IOException} TODO (kommentieren)
	 */
	public String writeHTML(final HTMLDocument doc, HTMLEditorKit kit) throws IOException, BadLocationException {
		StringWriter sw = new StringWriter();
		kit.write(sw, doc, 0, doc.getLength() - 1);
		final String string = sw.toString();
		Kostenrechner.getLogger().info(string);
		return string;
	}
}
//for (int index = 0; index < elem.getElementCount(); index++) {
//Element element = elem.getElement(index);
//System.out.println(element.toString() + element.getElementCount());
//doc.removeElement(element);
//}
//doc.insertAfterStart
// doc.getEndPosition().getOffset()
//doc.remove(0, doc.getLength());

//doc.insertString(0, throwable.getMessage() + "\n", doc.getStyle("bold"));
//doc.insertString(doc.getEndPosition().getOffset() - 1, throwable.getLocalizedMessage() + "\n",
//doc.getStyle("base"));
//doc.insertString(doc.getEndPosition().getOffset() - 1, String.format("%-20s%s%n", "", ste.toString()),
//doc.getStyle("base"));