/**
 * TenorDatenContainer.java
 * eu.gronos.kostenrechner.controller (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.tenordaten;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import eu.gronos.kostenrechner.interfaces.Tenorierend;
import eu.gronos.kostenrechner.interfaces.UnterContainerKlasse;

/**
 * Containerklasse, um alle Entscheidungstexte eines {@link Tenorierend} wie
 * auch dessen strukturierten Berechnungsdaten aufnehmen und für Debug- und
 * Support-Zwecke als XML speichern zu können
 * 
 * @author Peter Schuster (setrok)
 * @date 02.07.2018
 *
 */
@XmlRootElement(name = "SchustersStrukturierteTenordaten", namespace = "https://www.kostentenor.de/SchustersStrukturierteTenordaten")
public class TenorDatenContainer extends VerfahrensDatenContainer {

	public static class HauptsacheEntscheidungsElemente {
		@XmlElement(name = "HauptsacheentscheidungsText")
		public String text;

		@XmlElement(name = "hauptsacheVerurteilung")
		@XmlElementWrapper(name = "hauptsacheVerurteilungen")
		public List<HauptsacheVerhaeltnis> prozessverhaeltnisse = new ArrayList<HauptsacheVerhaeltnis>();
	}

	public static class KostenGrundEntscheidungsElemente {
		@XmlElement(name = "kostengrundentscheidungsText")
		public String text;

		@XmlElement(name = "kostenTragung")
		@XmlElementWrapper(name = "kostenTragungsEntscheidungen")
		public List<KostenTragungsVerhaeltnis> prozessverhaeltnisse = new ArrayList<KostenTragungsVerhaeltnis>();
	}

	public static class VollstreckbarkeitsEntscheidungsElemente {
		@XmlElement(name = "vollstreckbarkeitsEntscheidungsText")
		public String text;

		@XmlElement(name = "vollstreckungsVerhaeltnis")
		@XmlElementWrapper(name = "vollstreckungsVerhaeltnisse")
		public List<VollstreckungsVerhaeltnis> prozessverhaeltnisse = new ArrayList<VollstreckungsVerhaeltnis>();
	}

	public static class StreitwertEntscheidungsElemente {
		@XmlElement(name = "streitwertEntscheidungText")
		public String text;

		@XmlElement(name = "streitwert")
		@XmlElementWrapper(name = "streitwerte")
		public List<Long> streitwerte = new ArrayList<Long>();
	}

	public static class SonstigeEntscheidungsElemente {
		@XmlElement(name = "sonstigerEntscheidungsText")
		public String text;
	}

	/**
	 * Tabulierend sieht hier Object[][] tableCellValues vor, der von Class<?>[]
	 * columnClasses und String[] columnHeaders erklärt wird. Das lässt sich wohl
	 * auf XML schlecht übertragen... Aber es gibt eine begrenzte Anzahl möglicher
	 * Klassen, aus denen die Begründung zusammen gesetzt sein kann, nämlich String
	 * für die Zeilen-/Spalten-Überschriften und Double für die Inhalte.
	 */
	public static class BegruendungsElemente {
		@XmlElement(name = "begruendungText")
		public String text;

		@XmlElement(name = "caret")
		@XmlElementWrapper(name = "range")
		public int[] range = new int[2];

		@XmlElement(name = "spaltenUeberschrift")
		@XmlElementWrapper(name = "spaltenUeberschriften")
		public List<String> columnHeaders;

		@XmlElement(name = "zeilenUeberschrift")
		@XmlElementWrapper(name = "zeilenUeberschriften")
		public List<String> rowHeaders;

		@XmlElementWrapper(name = "zeilen")
		@XmlElement(name = "zeile")
		public List<DoubleDataRows> doubleData;
	}

	@XmlElement
	public String berechnungsTyp;

//	@XmlElement(name = "programminfo")
//	public Dateikopf dateikopf = new Dateikopf();

	@XmlElement(name = "hauptsache")
	public HauptsacheEntscheidungsElemente hauptsacheEntscheidung;
	// = new HauptsacheEntscheidungsElemente();

	@XmlElement(name = "kosten")
	public KostenGrundEntscheidungsElemente kostenEntscheidung;
	// = new KostenGrundEntscheidungsElemente();

	@XmlElement(name = "vorlaeufigVollstreckbar")
	public VollstreckbarkeitsEntscheidungsElemente vollstreckbarkeitsEntscheidung;
	// = new VollstreckbarkeitsEntscheidungsElemente();

	@XmlElement(name = "streitwertEntscheidung")
	public StreitwertEntscheidungsElemente streitwertEntscheidung;
	// = new StreitwertEntscheidungsElemente();

	@XmlElement(name = "sonstiges")
	public SonstigeEntscheidungsElemente sonstigeEntscheidung;
	// = new SonstigeEntscheidungsElemente();

	@XmlElement(name = "begruendung")
	public BegruendungsElemente begruendung;
	// = new BegruendungsElemente();

	/**
	 * Standard-Konstruktor ohne Parameter, für {@link JAXB}
	 * 
	 */
	public TenorDatenContainer() {
		super();
		dateikopf = new Dateikopf();
		hauptsacheEntscheidung = new HauptsacheEntscheidungsElemente();
		kostenEntscheidung = new KostenGrundEntscheidungsElemente();
		vollstreckbarkeitsEntscheidung = new VollstreckbarkeitsEntscheidungsElemente();
		streitwertEntscheidung = new StreitwertEntscheidungsElemente();
		sonstigeEntscheidung = new SonstigeEntscheidungsElemente();
		begruendung = new BegruendungsElemente();
	}

	/**
	 * Konstruktor, der einen vollständigen {@link VerfahrensDatenContainer}
	 * erweitert.
	 * 
	 * @param container ein {@link VerfahrensDatenContainer}
	 */
	public TenorDatenContainer(VerfahrensDatenContainer container) {
		this();
		this.allgemein = container.allgemein;
		this.teilklageruecknahme = container.teilklageruecknahme;
		this.streitgenossen = container.streitgenossen;
		this.staffelung = container.staffelung;
		this.gebuehrenBerechnung = container.gebuehrenBerechnung;
	}

	/**
	 * Konstruktor, der die {@link UnterContainerKlasse}n eines
	 * {@link VerfahrensDatenContainer} übernimmt.
	 * 
	 * @param unters mehrere {@link UnterContainerKlasse}n
	 */
	public TenorDatenContainer(UnterContainerKlasse... unters) {
		this();
		for (UnterContainerKlasse unter : unters) {
			if (unter instanceof Allgemein)
				this.allgemein = (Allgemein) unter;
			else if (unter instanceof Teilklageruecknahme)
				this.teilklageruecknahme = (Teilklageruecknahme) unter;
			else if (unter instanceof Streitgenossen)
				this.streitgenossen = (Streitgenossen) unter;
			else if (unter instanceof ForderungsStaffelung)
				this.staffelung = (ForderungsStaffelung) unter;
			else if (unter instanceof GebuehrenBerechnung)
				this.gebuehrenBerechnung = (GebuehrenBerechnung) unter;
		}
	}
}
