/*
 * KostenFileChooser.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import eu.gronos.kostenrechner.controller.system.StringConfigOption;

/**
 * Die Klasse dient dazu, einen {@link JFileChooser} zu erzeugen, der fürs
 * Speichern von RTF- und XML-Dateien vorkonfiguriert ist.
 *
 * @author Peter Schuster (setrok)
 * @date 9 Jun 2019
 *
 */
public abstract class KostenFileChooser extends JFileChooser {
	private static File speicherOrt;
	private static StringConfigOption speicherOrtOption = new StringConfigOption("speicherOrt", "");

	private static final long serialVersionUID = -8758824500337543760L;

	/**
	 * Erzeugt einen JFileChooser, der fürs Speichern von Dateien des übergebenen
	 * {@link FileFilter}s vorkonfiguriert ist.
	 * 
	 * @param filter ein {@link FileFilter}
	 */
	public KostenFileChooser(FileFilter filter) {
		super();
		setMultiSelectionEnabled(false);
		setDialogTitle(String.format("Datei im Format %s wählen...", filter.getDescription()));
		/*
		 * Den "alle Dateien" Filter muss man wohl entfernen, bevor man den gewünschten
		 * Filter hinzufügt.
		 */
		setAcceptAllFileFilterUsed(false);
		addChoosableFileFilter(filter);
		setFileFilter(filter);
		setCurrentDirectory(getSpeicherOrt());
	}

	/**
	 * Die Methode poppt einen Datei-Speicher-Dialog auf. Sie kümmert sich darum,
	 * dass der eingegebene Dateiname die Endung aus
	 * {@link KostenFileChooser#KostenFileChooser(FileFilter)} bekommt. Sie warnt
	 * auch, wenn eine Datei desselben Namens schon existiert und lässt den Nutzer
	 * reagieren. Der Speicherpfad wird gespeichert.
	 * 
	 * @param parent the parent component of the dialog, can be <code>null</code>;
	 *               see <code>showDialog</code> for details
	 * @return the return state of the file chooser on popdown:
	 *         <ul>
	 *         <li>JFileChooser.CANCEL_OPTION
	 *         <li>JFileChooser.APPROVE_OPTION
	 *         <li>JFileChooser.ERROR_OPTION if an error occurs or the dialog is
	 *         dismissed
	 *         </ul>
	 * 
	 * @see javax.swing.JFileChooser#showSaveDialog(java.awt.Component)
	 */
	public int showSaveDialog(Component parent) {//
		setCurrentDirectory(getSpeicherOrt());
		int option = super.showSaveDialog(parent);
		File datei = null;
		if (option == JFileChooser.APPROVE_OPTION) {
			datei = getSelectedFile().getAbsoluteFile();
			if (!datei.getName().endsWith(baueGanzeEndung())) {
				datei = new File(datei.getAbsolutePath() + baueGanzeEndung());
				setSelectedFile(datei);
			}
			if (datei.exists()) {
				int bestaetigung = JOptionPane.showConfirmDialog(this, "Die Datei existiert schon. \n\nFortfahren?");
				if (bestaetigung == JOptionPane.OK_OPTION) {
					//
				} else {
					JOptionPane.showMessageDialog(this, "Die Datei wurde nicht gespeichert.");
					return JFileChooser.CANCEL_OPTION;
				}
			}
			// jetzt den Pfad *merken*!
			setSpeicherOrt(getCurrentDirectory());
		}
		return option;
	}

	/**
	 * Die Methode dient dazu, einen Dateiauswahldialog der {@link JFileChooser}
	 * Oberklasse aufzurufen. Der Speicherpfad wird gespeichert.
	 * 
	 * @param parent
	 * @return the return state of the file chooser on popdown:
	 *         <ul>
	 *         <li>JFileChooser.CANCEL_OPTION
	 *         <li>JFileChooser.APPROVE_OPTION
	 *         <li>JFileChooser.ERROR_OPTION if an error occurs or the dialog is
	 *         dismissed
	 *         </ul>
	 * 
	 * @see javax.swing.JFileChooser#showOpenDialog(java.awt.Component)
	 */
	public int showOpenDialog(Component parent) {
		setCurrentDirectory(getSpeicherOrt());
		int option = super.showOpenDialog(parent);
		if (option == JFileChooser.APPROVE_OPTION) {
			// jetzt den Pfad *merken*!
			setSpeicherOrt(getCurrentDirectory());
		}
		return option;
	}

	/**
	 * Die Methode gibt die vollständige Dateiendung des voreingestellten
	 * {@link FileFilter}s zurück, d.h. mit Punkt "."
	 * 
	 * @return einen {@link String} "." + Dateiendung
	 *         {@link FileNameExtensionFilter#getExtensions()}
	 */
	private String baueGanzeEndung() {
		final FileFilter filter = getFileFilter();
		if (filter instanceof FileNameExtensionFilter) {
			return "." + ((FileNameExtensionFilter) filter).getExtensions()[0];
		}
		return "";
	}
//		else if (filter instanceof RtfFileFilter) {return RtfFileFilter.RTF_EXTENSION;		}

	/**
	 * Die Methode gibt den {@link #speicherOrt} zurück, also das Verzeichnis, in
	 * eine Unterklasse zuletzt eine Datei gespeichert bzw. aus dem sie zuletzt eine
	 * Datei geladen hat. Liest aus Registry!
	 * 
	 * @return gibt {@link #speicherOrt} als {@link File} zurück.
	 */
	protected File getSpeicherOrt() {
		if (speicherOrt == null) {
			speicherOrt = new File(speicherOrtOption.getValue());
			if (!speicherOrt.exists() || !speicherOrt.isDirectory()) {
				speicherOrtOption.reset();
				speicherOrt = new File(speicherOrtOption.getValue());
			}
//			return new File("");
		}
//		System.out.println(speicherOrt);
		return speicherOrt;
	}

	/**
	 * Setzt {@link #getSpeicherOrt()} und speichert die Zeichenkette in der
	 * Registry!
	 * 
	 * @param pfad d. {@link #speicherOrt}, d. gesetzt werden soll als
	 *             {@link String}.
	 */
	protected void setSpeicherOrt(File pfad) {
		speicherOrt = pfad;
		speicherOrtOption.setValue(speicherOrt.getAbsolutePath());
	}

}
